@@ -1,9 +1,5 @@
-# Author: Oscar Perpinan Lamigueiro oscar.perpinan@upm.es
-# Date :  June 2011
-# Version 0.10
-# Licence GPL v3
-
 setGeneric('densityplot')
+
 setMethod('densityplot',
           signature(x='RasterLayer', data='missing'),
           definition=function (x, data=NULL, maxpixels = 1e+05,
@@ -61,9 +57,7 @@ setMethod('densityplot', signature(x='formula', data='Raster'),
             auto.key = list(space = 'right'), 
             par.settings=rasterTheme(),...){
 
-            ## names replace layerNames with raster version 2.0-04
-            rasterVersion <- as.character(packageVersion('raster'))
-            nms <- if (compareVersion(rasterVersion, '2.0-04') == -1) layerNames(data) else names(data)
+            nms <- names(data)
 
             nl <- nlayers(data)
 
